/*
 * This file is part of the GROMACS molecular simulation package.
 *
 * Copyright 2019- The GROMACS Authors
 * and the project initiators Erik Lindahl, Berk Hess and David van der Spoel.
 * Consult the AUTHORS/COPYING files and https://www.gromacs.org for details.
 *
 * GROMACS is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * GROMACS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with GROMACS; if not, see
 * https://www.gnu.org/licenses, or write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * If you want to redistribute modifications to GROMACS, please
 * consider that scientific software is very special. Version
 * control is crucial - bugs must be traceable. We will be happy to
 * consider code for inclusion in the official distribution, but
 * derived work must not be called official GROMACS. Details are found
 * in the README & COPYING files - if they are missing, get the
 * official version at https://www.gromacs.org.
 *
 * To help us fund GROMACS development, we humbly ask that you cite
 * the research papers on the package. Check out https://www.gromacs.org.
 */
/*! \internal \file
 * \brief Defines the composite element for the modular simulator
 *
 * \author Pascal Merz <pascal.merz@me.com>
 * \ingroup module_modularsimulator
 */

#include "gmxpre.h"

#include "compositesimulatorelement.h"

#include <utility>

#include "gromacs/mdlib/stat.h"
#include "gromacs/modularsimulator/modularsimulatorinterfaces.h"

namespace gmx
{
CompositeSimulatorElement::CompositeSimulatorElement(
        std::vector<compat::not_null<ISimulatorElement*>>    elementCallList,
        std::vector<std::unique_ptr<gmx::ISimulatorElement>> elements,
        int                                                  frequency) :
    elementCallList_(std::move(elementCallList)),
    elementOwnershipList_(std::move(elements)),
    frequency_(frequency)
{
}

void CompositeSimulatorElement::scheduleTask(Step step, Time time, const RegisterRunFunction& registerRunFunction)
{
    if (do_per_step(step, frequency_))
    {
        for (auto& element : elementCallList_)
        {
            element->scheduleTask(step, time, registerRunFunction);
        }
    }
}

void CompositeSimulatorElement::elementSetup()
{
    for (auto& element : elementOwnershipList_)
    {
        element->elementSetup();
    }
}

void CompositeSimulatorElement::elementTeardown()
{
    for (auto& element : elementOwnershipList_)
    {
        element->elementTeardown();
    }
}

} // namespace gmx
