/*
 * This file is part of the GROMACS molecular simulation package.
 *
 * Copyright 1991- The GROMACS Authors
 * and the project initiators Erik Lindahl, Berk Hess and David van der Spoel.
 * Consult the AUTHORS/COPYING files and https://www.gromacs.org for details.
 *
 * GROMACS is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * GROMACS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with GROMACS; if not, see
 * https://www.gnu.org/licenses, or write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * If you want to redistribute modifications to GROMACS, please
 * consider that scientific software is very special. Version
 * control is crucial - bugs must be traceable. We will be happy to
 * consider code for inclusion in the official distribution, but
 * derived work must not be called official GROMACS. Details are found
 * in the README & COPYING files - if they are missing, get the
 * official version at https://www.gromacs.org.
 *
 * To help us fund GROMACS development, we humbly ask that you cite
 * the research papers on the package. Check out https://www.gromacs.org.
 */

#ifndef GMX_GMXPREPROCESS_VSITE_PARM_H
#define GMX_GMXPREPROCESS_VSITE_PARM_H

class PreprocessingAtomTypes;
struct gmx_moltype_t;
struct t_atoms;
struct InteractionsOfType;

#include "gromacs/topology/ifunc.h"
#include "gromacs/utility/enumerationhelpers.h"

namespace gmx
{
template<typename>
class ArrayRef;
class MDLogger;
} // namespace gmx

int set_vsites(bool                                                            bVerbose,
               t_atoms*                                                        atoms,
               PreprocessingAtomTypes*                                         atype,
               gmx::EnumerationArray<InteractionFunction, InteractionsOfType>& plist,
               const gmx::MDLogger&                                            logger);
/* set parameters for virtual sites, return number of virtual sites */

void set_vsites_ptype(bool bVerbose, gmx_moltype_t* molt, const gmx::MDLogger& logger);
/* set ptype to VSite for virtual sites */

/*! \brief Clean up the bonded interactions
 *
 * Throw away all obsolete bonds, angles and dihedrals.
 * Throw away all constraints. */
void clean_vsite_bondeds(gmx::EnumerationArray<InteractionFunction, InteractionsOfType>& ps,
                         int                                                             natoms,
                         bool                 bRmVSiteBds,
                         const gmx::MDLogger& logger);

#endif
